package com.dragoninfo.dcuc.authweb.exception;


import com.dragoninfo.dcuc.common.enums.ErrorCodeEnum;
import com.dragoninfo.dcuc.common.exception.GmConfidentialityException;
import com.dragoninfo.dcuc.common.exception.GmIntegrityException;
import com.dragoninfo.dcuc.common.exception.GmNonRepudiationException;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import feign.FeignException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.servlet.HandlerExceptionResolver;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;
import java.util.Objects;

/**
 * 错误时异常转换
 *
 * @author huangzqa
 * @date 2021/2/28
 **/
@Component
@Order(-100)
public class RespExceptionHandler implements HandlerExceptionResolver {

    private static final Logger log = LoggerFactory.getLogger(RespExceptionHandler.class);

    @Override
    public ModelAndView resolveException(HttpServletRequest request, HttpServletResponse response, Object o, Exception ex) {
        log.error("异常", ex);
        try {
            response.setContentType("application/json;charset=UTF-8");
            response.setStatus(500);
            PrintWriter writer = response.getWriter();
            ResponseStatus fail = null;
            if (ex instanceof ConstraintViolationException) {
                response.setStatus(HttpStatus.INTERNAL_SERVER_ERROR.value());
                fail = new ResponseStatus(String.valueOf(ErrorCodeEnum.REQUEST_PARAM.getValue()), String.format(ErrorCodeEnum.REQUEST_PARAM.getLabel(), ex.getMessage()));
            } else if (ex instanceof MethodArgumentNotValidException) {
                response.setStatus(HttpStatus.INTERNAL_SERVER_ERROR.value());
                String msg = getValidMessage((MethodArgumentNotValidException) ex);
                fail = new ResponseStatus(String.valueOf(ErrorCodeEnum.REQUEST_PARAM.getValue()), String.format(ErrorCodeEnum.REQUEST_PARAM.getLabel(), msg));
            } else if (ex instanceof BindException) {
                response.setStatus(HttpStatus.INTERNAL_SERVER_ERROR.value());
                String message = Objects.requireNonNull(((BindException) ex).getFieldError()).getDefaultMessage();
                fail = new ResponseStatus(String.valueOf(ErrorCodeEnum.REQUEST_PARAM.getValue()), String.format(ErrorCodeEnum.REQUEST_PARAM.getLabel(), message));
            } else if (ex instanceof FeignException) {
                String message = ex.getMessage();
                if (message.contains(GmConfidentialityException.ERROR_MESSAGE)) {
                    fail = ResponseStatus.fail(String.valueOf(HttpStatus.NOT_ACCEPTABLE.value()), GmConfidentialityException.ERROR_MESSAGE);
                } else if (message.contains(GmIntegrityException.ERROR_MESSAGE)) {
                    fail = ResponseStatus.fail(String.valueOf(HttpStatus.NOT_ACCEPTABLE.value()), GmIntegrityException.ERROR_MESSAGE);
                } else if (message.contains(GmNonRepudiationException.ERROR_MESSAGE)) {
                    fail = ResponseStatus.fail(String.valueOf(HttpStatus.NOT_ACCEPTABLE.value()), GmNonRepudiationException.ERROR_MESSAGE);
                } else {
                    fail = ResponseStatus.fail("500", "未知异常[" + ex.getMessage() + "]");
                }
            } else {
                fail = ResponseStatus.fail("500", "未知异常[" + ex.getMessage() + "]");
            }

            writer.write(JsonUtils.toJSONString(fail));
            writer.flush();
        } catch (IOException var8) {
            log.error(var8.getMessage(), var8);
        }

        return new ModelAndView();
    }


    /**
     * 解析Valid错误信息
     *
     * @param ex
     * @return
     */
    private String getValidMessage(MethodArgumentNotValidException ex) {
        List<ObjectError> list = ex.getBindingResult().getAllErrors();
        StringBuilder msg = new StringBuilder();
        for (int i = 0; i < list.size(); i++) {
            ObjectError error = list.get(i);
            if (list.size() == 1) {
                msg.append("错误信息：").append(((FieldError) error).getField()).append(error.getDefaultMessage()).append("；");
            } else {
                msg.append("错误信息").append(i + 1).append("：").append(((FieldError) error).getField()).append(error.getDefaultMessage()).append("；");
            }
        }
        return msg.toString();
    }
}
